#include <allegro5/allegro.h>
#include <allegro5/allegro_image.h>

int main(void) {

  bool render = false;
  bool running = true;

  // Camera vars
  float camera_x = 0.0;
  float camera_y = 0.0;

  float smile_x = 0.0;
  float smile_y = 0.0;

  // Excuse the lack of error checking

  al_init();

  al_set_new_display_flags(ALLEGRO_WINDOWED);

  ALLEGRO_TIMER *timer = al_create_timer(1.0 / 60.0);
  ALLEGRO_DISPLAY *display = al_create_display(480, 480);
  ALLEGRO_EVENT_QUEUE *event_queue = al_create_event_queue();

  // Give the window a fancy display title
  al_set_window_title(display, "Example");

  al_init_image_addon();

  al_register_event_source(event_queue, al_get_timer_event_source(timer));
  al_register_event_source(event_queue, al_get_display_event_source(display));

  // Load our beautiful bitmaps
  ALLEGRO_BITMAP *smile = al_load_bitmap("smile.png");
  ALLEGRO_BITMAP *grass = al_load_bitmap("grass.png");

  al_start_timer(timer);

  // Stretch display
  ALLEGRO_TRANSFORM t;
  al_identity_transform(&t);
  al_scale_transform(&t, 5, 5); // Works OK if scale value is > 5
  al_use_transform(&t);

  // Primary game loop
  while (running) {

    ALLEGRO_EVENT event;

    al_wait_for_event(event_queue, &event);

    switch (event.type) {

      case ALLEGRO_EVENT_DISPLAY_CLOSE:

        // Shut the program down
        running = false;
      break;

      case ALLEGRO_EVENT_TIMER:

        render = true;

        // Crappy camera calculations
        camera_x = smile_x;
        camera_y = smile_y;

        // Move player down screen
        smile_y += 0.5;
      break;
    }

    // Drawing phase
    if (render && al_is_event_queue_empty(event_queue)) {

      render = false;

      if (smile && grass) {

        al_hold_bitmap_drawing(true);

        for (int y = 0; y < 100; ++y) {

          for (int x = 0; x < 100; ++x) {

            // Draw grass tiles
            al_draw_bitmap(grass, x * 8 - camera_x, y * 9 - camera_y, 0);
          }
        }

        al_hold_bitmap_drawing(false);

        // Draw our smile guy (think of him as player)
        al_draw_bitmap(smile, smile_x - camera_x, smile_y - camera_y, 0);
      }

      al_flip_display();
      al_clear_to_color(al_map_rgb(0, 0, 255));
    }
  }

  // Garbage collection
  al_destroy_timer(timer);
  al_destroy_display(display);
  al_destroy_event_queue(event_queue);

  return 0;
}
